function f = get(P,field)

% GET - extract field of matrix polynomial object
% 
%        f = get(P,'field')
%
% This is the same as 
%
%        f = P.field
%
% Allowable fields are
%        min    = smallest exponent
%        max    = largest exponent
%        coef   = matrix of coefficients
%        length = max - min + 1 = number of coefficients
%        degree = polynomial degree = length - 1
%        size   = size of coefficient matrices
%        type   = '', 'polyphase' or 'symbol'
%        m      = dilation factor 
%        r      = multiplicity

% Actually, "f = P.field" is not quite the same as "f = get(P,'field')".
% They are the same if you call them from anywhere outside the @mpoly directory.
% However, inside @mpoly, "f = P.field" calls the system subscripting routine,
% so it only works for the fields that are actually stored. The GET method 
% works even in @mpoly.

% Copyright (c) 2004 by Fritz Keinert (keinert@iastate.edu),
% Dept. of Mathematics, Iowa State University, Ames, IA 50011.
% This software may be freely used and distributed for non-commercial
% purposes, provided this copyright statement is preserved, and
% appropriate credit for its use is given.
%
% Last update: Feb 20, 2004

switch field
% max, length, degree, size need to be computed   
 case 'max'
  f = P.min + size(P.coef,3) - 1;
 case 'length'
  f = size(P.coef,3);
 case 'degree'
  f = size(P.coef,3) - 1;
 case 'size'
  f = [size(P.coef,1),size(P.coef,2)];
 otherwise
% min, coef, type, m, r are actually stored
  f = eval(['P.',field]);
end
