function P = reshape(P,n1,n2)

% RESHAPE -- built-in RESHAPE, extended to matrix polynomials
%
%        P = reshape(P,n1,n2)
%
% This reshapes P.coef into size n1 x n2 x n3, where n3 is chosen
% to keep the product of the dimensions invariant. This command
% can be used to reshape the size of each coefficient, or convert
% between symbol and polyphase form.
%
% n1 x n2 must divide the number of entries in P evenly.

% Copyright (c) 2004 by Fritz Keinert (keinert@iastate.edu),
% Dept. of Mathematics, Iowa State University, Ames, IA 50011.
% This software may be freely used and distributed for non-commercial
% purposes, provided this copyright statement is preserved, and
% appropriate credit for its use is given.
%
% Last update: Feb 20, 2004

[m1,m2,m3] = size(P.coef);
n3 = round(m1*m2*m3/(n1*n2));
if (n1*n2*n3 ~= m1*m2*m3)
    error('dimensions are not compatible');
end
P.coef = reshape(P.coef,n1,n2,n3);
